# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

######################### We start with some black magic to print on failure.

# Change 1..1 below to 1..last_test_to_print .
# (It may become useful if the test is moved to ./t subdirectory.)

BEGIN { $| = 1; print "1..3\n"; }
END {print "not ok 1\n" unless $loaded;}
use ex::override;
$loaded = 1;
print "ok 1\n";

sub ok {
  my( $number, $value ) = @_;
  print $value ? "ok $number\n" : "not ok $number\n";
}

######################### End of black magic.

# Insert your test code below (better if it prints "ok 13"
# (correspondingly "not ok 13") depending on the success of chunk 13
# of the test code):

package GlobalOverride;

use ex::override GLOBAL_length => sub { length( join '', @_ ) };

sub goodlength {
  no ex::override 'GLOBAL_length';
  length( $_[0] );
}

1;

package main;
our @ISA = 'GlobalOverride';

ok( 2, length( 'One', 'Two', 'Three' ) == 11 );


ok( 3, &GlobalOverride::goodlength( 'One', 'Two', 'Three' ) == 3 );

